<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="xml" encoding="UTF-8"
    doctype-system="http://www.springframework.org/dtd/spring-beans.dtd"
    doctype-public="-//SPRING//DTD BEAN//EN"
    cdata-section-elements="beans" />

  <!-- Default stage table names -->
	<xsl:variable name="ST_USERS">DS_USERS</xsl:variable>

  <!-- Staging Field names -->
	<xsl:variable name="role_list_field">role_list</xsl:variable>
	<xsl:variable name="person_supervisor_list_field">person_supervisor_list</xsl:variable>
	<xsl:variable name="language_override_field">language_override</xsl:variable>
	<xsl:variable name="timezone_override_field">timezone_override</xsl:variable>
	<xsl:variable name="web_login_type_field">web_login_type</xsl:variable>
	<xsl:variable name="site_name_field">site_name</xsl:variable>
	<xsl:variable name="ldap_domain_field">ldap_domain</xsl:variable>
  <xsl:variable name="web_password_field">web_password</xsl:variable>
  <xsl:variable name="phone_password_field">phone_password</xsl:variable>
  <xsl:variable name="has_mobile_access_field">has_mobile_access</xsl:variable>
  
	<!-- Row Synchronizers -->
	<xsl:variable name="person_row_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.EnterprisePersonRowSynchronizerImpl</xsl:variable>
  <xsl:variable name="person_supervisor_row_synchronizer">com.invoqsystems.apex.datasynchronizer.processor.EnterpriseUserSupervisorRowSynchronizerImpl</xsl:variable>
  <xsl:variable name="staging_person_supervisor_rsync">com.invoqsystems.apex.datasynchronizer.processor.StagingUserSupervisorRowSynchronizerImpl</xsl:variable>

  <!-- Users .. Table Level -->
	<xsl:template match="user">
		<bean>
			<xsl:attribute name="class">
        <xsl:call-template name="choose_table_synchronizer_class"/>
			</xsl:attribute>
			<xsl:call-template name="inject_staging_datasource" />
			<xsl:call-template name="choose_staging_table_name">
			  <xsl:with-param name="default_name" select="$ST_USERS"/>
			</xsl:call-template>
			<xsl:call-template name="include_audit_service" />
			<property name="dataSynchronizationService">
				<ref bean="dataSyncPersonService" />
			</property>
			<!-- Row Synchronizer -->
			<xsl:call-template name="row_synchronizer">
				<xsl:with-param name="sync_class" select="$person_row_synchronizer"/>
				<xsl:with-param name="data_synchronization_service">dataSyncPersonService</xsl:with-param>
				<xsl:with-param name="default_table" select="$ST_USERS" />
			</xsl:call-template>
			<xsl:call-template name="inject_report_keeper"/>
		</bean>
	</xsl:template>

  <!-- Field Level -->
	<!-- Note that if no mapper class is specified then a match will be done on type, the default mapper is String -->
	<xsl:template match="first_name">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">firstName</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="last_name">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">lastName</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="user_id">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">targetName</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="site_name">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">currentSite</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$referenceKeyTypeMapper" />
      <xsl:with-param name="mapper_properties">dataSyncSiteService</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="role_list">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="synchronizer_class" select="$enterprise_list_field_synchronizer"/>
			<xsl:with-param name="model_property_name">administrativeGroups</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$listTypeMapper" />
      <xsl:with-param name="validatorBean">nonEmptySetFieldValidator</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="language_override">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">preferredLanguage</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$referenceKeyTypeMapper" />
      <xsl:with-param name="mapper_properties">dataSyncLanguageService</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="timezone_override">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">preferredTimezone</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$referenceKeyTypeMapper" />
      <xsl:with-param name="mapper_properties">dataSyncTimeZoneService</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="recorded_name">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">recordedName</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="phone_login">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">phoneLogin</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="phone_password">
		<xsl:call-template name="call_fields">
      <xsl:with-param name="synchronizer_class" select="$password_synchronizer_class"/>
			<xsl:with-param name="model_property_name">phonePassword</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="web_login_type">
		<xsl:param name="tableName" />
		<xsl:call-template name="call_fields">
			<xsl:with-param name="tableName" select="$tableName" />
			<xsl:with-param name="synchronizer_class" select="$enterprise_web_login_type_field_synchronizer"/>
			<xsl:with-param name="model_property_name">webCredentialType</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$webLoginTypeTypeMapper" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="web_login">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">webLogin</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="web_password">
		<xsl:call-template name="call_fields">
      <xsl:with-param name="synchronizer_class" select="$password_synchronizer_class"/>
			<xsl:with-param name="model_property_name">webPassword</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="ldap_domain">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">LDAPDomain</xsl:with-param>
			<xsl:with-param name="mapper_class" select="$referenceKeyTypeMapper" />
      <xsl:with-param name="mapper_properties">dataSyncLDAPDomainService</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="has_mobile_access">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">hasMobileAccess</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$booleanTypeMapper" />
		</xsl:call-template>
	</xsl:template>

  <!-- Joined Tables -->
  <xsl:variable name="ST_USER_JOIN_ATTRIBUTE">DS_USER_JOIN_ATTRIBUTES</xsl:variable> 

  <!-- Joined Table User Attributes -->
  <xsl:variable name="user_attributes_field">user_attributes</xsl:variable>
  <xsl:template match="user_attributes">
    <entry>
      <xsl:attribute name="key">
        <xsl:value-of select="name()" />
      </xsl:attribute>
      <bean>
        <xsl:call-template name="core_field_properties">
          <xsl:with-param name="synchronizer_class" select="$collection_fieldsync_class"/>
          <xsl:with-param name="model_property_name">customAttributes</xsl:with-param>
          <xsl:with-param name="mapper_class" select="$hashSetTypeMapper"/>
        </xsl:call-template>

        <property name="joinedTableSynchronizer">
          <bean>
            <xsl:call-template name="joined_table_synchronizer">
              <xsl:with-param name="joined_table_name" select="$ST_USER_JOIN_ATTRIBUTE"/>
              <xsl:with-param name="parent_external_key">USER_EXTERNAL_KEY</xsl:with-param>
              <xsl:with-param name="model_property_name">externalKey</xsl:with-param>
              <xsl:with-param name="child_external_key">ATTRIBUTE_EXTERNAL_KEY</xsl:with-param>
              <xsl:with-param name="mapper_class" select="$referenceKeyTypeMapper" />
              <xsl:with-param name="mapper_properties">dataSyncCustomAttributeService</xsl:with-param>
            </xsl:call-template>
          </bean>
        </property>
      </bean>
    </entry>
  </xsl:template>

  <xsl:template name="user_attributes_mapper_properties">
    <property name="dataSynchronizationService">
      <ref bean="dataSyncCustomAttributeService" />
    </property>
	</xsl:template>

  <!-- User Supervisors -->
  <xsl:variable name="ST_USER_SUPERVISORS">DS_PERSON_SUPERVISORS</xsl:variable> 
  <!-- Table Level -->
	<xsl:template match="person_supervisor">
		<bean>
			<xsl:attribute name="class">
        <xsl:call-template name="choose_table_synchronizer_class"/>
			</xsl:attribute>
			<xsl:call-template name="inject_staging_datasource" />
			<xsl:call-template name="choose_staging_table_name">
			  <xsl:with-param name="default_name" select="$ST_USER_SUPERVISORS"/>
			</xsl:call-template>
			<xsl:call-template name="include_audit_service" />
			<property name="dataSynchronizationService">
				<ref bean="dataSyncPersonService" />
			</property>
			<!-- Row Synchronizer -->
			<xsl:call-template name="row_synchronizer">
				<xsl:with-param name="sync_class" select="$person_supervisor_row_synchronizer"/>
				<xsl:with-param name="data_synchronization_service">dataSyncPersonService</xsl:with-param>
				<xsl:with-param name="default_table" select="$ST_USER_SUPERVISORS" />
				<xsl:with-param name="row_marker">rowStagingTableUserSupervisorMarker</xsl:with-param>
			</xsl:call-template>
			<xsl:call-template name="inject_report_keeper"/>
		</bean>
	</xsl:template>

	<xsl:template match="user_external_key">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="model_property_name">externalKey</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="person_supervisor_list">
		<xsl:call-template name="call_fields">
			<xsl:with-param name="synchronizer_class" select="$enterprise_list_field_synchronizer"/>
			<xsl:with-param name="model_property_name">supervisors</xsl:with-param>
      <xsl:with-param name="mapper_class" select="$listTypeMapper" />
		</xsl:call-template>
	</xsl:template>


</xsl:stylesheet>
